//*****************************************************************************
//* ACKme DRIVER FOR PIC32MX595F512H
//* WRITTEN BY FRED EADY
//* NO USB SUPPORT IN THIS VERSION
//* MPLABX 2.15 XC32 1.34
//* LAST UPDATE	07/16/2015
//* NOTES:
//*****************************************************************************

//*****************************************************************************
//* CONFIGURATION BITS
//*****************************************************************************
    #pragma config UPLLEN   = OFF           // USB PLL Disabled
    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier
    //#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    #pragma config FWDTEN   = OFF           // Watchdog Timer
    #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
    #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
//*****************************************************************************
//* Internal Function Prototypes
#include <xc.h>
#include <plib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "GenericTypeDefs.h"

typedef struct
{
	BYTE	falt:1;
}FFLAGS;
FFLAGS flags;

//*****************************************************************************
// PROTOTYPES
//*****************************************************************************
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
void UserInit(void);
void delayms(unsigned int ms);
void init(void);
BYTE recvchar2(void);
BYTE recvchar4(void);
BYTE CharInQueue2(void);
BYTE CharInQueue4(void);
BYTE recvchar5(void);
BYTE CharInQueue5(void);
BYTE enterCmdMode(void);
BYTE exitCmdMode(void);
BYTE reboot(void);
//*****************************************************************************
// MACROS
//*****************************************************************************
#define GetSystemClock()            80000000UL
#define GetPeripheralClock()        80000000UL
//#define GetInstructionClock()       (GetSystemClock() / 2)
#define CoreTicksPerMs              (GetSystemClock() / 2000)
//******************************************************************
//*	USART Definitions
//******************************************************************
#define USART_RX_BUFFER_SIZE2  256       /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define USART_RX_BUFFER_MASK2 ( USART_RX_BUFFER_SIZE2 - 1 )
BYTE USART_RxBuf2[USART_RX_BUFFER_SIZE2];
BYTE USART_RxHead2,USART_RxTail2;

#define USART_RX_BUFFER_SIZE4  256       /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define USART_RX_BUFFER_MASK4 ( USART_RX_BUFFER_SIZE4 - 1 )
BYTE USART_RxBuf4[USART_RX_BUFFER_SIZE4];
BYTE USART_RxHead4,USART_RxTail4;

#define USART_RX_BUFFER_SIZE5  256       /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define USART_RX_BUFFER_MASK5 ( USART_RX_BUFFER_SIZE5 - 1 )
BYTE USART_RxBuf5[USART_RX_BUFFER_SIZE5];
BYTE USART_RxHead5,USART_RxTail5;
//*****************************************************************************
//* SRAM ALLOCATIONS
//*****************************************************************************
BYTE scratch8;
BYTE bitein;
char rxBuf5[256];
char rxBuf2[256];
char rxBuf4[256];
char indx5,indx2,indx4;
//*****************************************************************************
// I/O PIN ALIASES
//*****************************************************************************
#define	btnUTL              (PORTFbits.RF3)

#define rstHI               LATBSET = 0x0020;    //0000 0000 0010 0000
#define rstLO               LATBCLR = 0x0020;    //0000 0000 0010 0000

#define atm1rstLO           LATBCLR = 0x8000;    //1000 0000 0000 0000
#define atm1rstHI           LATBSET = 0x8000;    //1000 0000 0000 0000

#define gpio9LO             LATECLR = 0x0200;    //0000 0000 0010 0000
#define gpio9HI             LATESET = 0x0200;    //0000 0000 0010 0000
/** LEDS ************************************************************/
#define bluLED             LATEbits.LATE4
#define bluLED_Off         LATECLR = 0x0010;   //0000 0000 0001 0000
#define bluLED_On          LATESET = 0x0010;   //0000 0000 0001 0000
#define bluLED_Toggle      LATEINV = 0x0010;   //0000 0000 0001 0000

#define redLED             LATEbits.LATE3
#define redLED_Off         LATECLR = 0x0008;   //0000 0000 0000 1000
#define redLED_On          LATESET = 0x0008;   //0000 0000 0000 1000
#define redLED_Toggle      LATEINV = 0x0008;   //0000 0000 0000 1000

#define grnLED             LATEbits.LATE2
#define grnLED_Off         LATECLR = 0x0004;   //0000 0000 0000 0100
#define grnLED_On          LATESET = 0x0004;   //0000 0000 0000 0100
#define grnLED_Toggle      LATEINV = 0x0004;   //0000 0000 0000 0100
//*****************************************************************************
// UART 2 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_2_VECTOR, IPL2SOFT) IntUart2Handler(void)
{
	BYTE tmphead2;
        BYTE rxdata2;

  // Is this an RX interrupt?
  if (INTGetFlag(INT_SOURCE_UART_RX(UART2)))
    {
        rxdata2 = UARTGetDataByte(UART2);
        // calculate buffer index
        tmphead2 = ( USART_RxHead2 + 1 ) & USART_RX_BUFFER_MASK5;
        // store new index
        USART_RxHead2 = tmphead2;
        // if error
        if ( tmphead2 == USART_RxTail2 )
        {
            UARTEnable(UART2, UART_DISABLE_FLAGS( UART_RX));
            UARTEnable(UART2, UART_ENABLE_FLAGS(UART_RX));
        }
        //store received data in buffer
        USART_RxBuf2[tmphead2] = rxdata2;

      // Clear the RX interrupt Flag
      INTClearFlag(INT_SOURCE_UART_RX(UART2));

      // Toggle LED to indicate UART activity
      bluLED_Toggle;
    }

  // We don't care about TX interrupt
  if ( INTGetFlag(INT_SOURCE_UART_TX(UART2)) )
    {
      INTClearFlag(INT_SOURCE_UART_TX(UART2));
    }
}
//*****************************************************************************
// UART 4 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_4_VECTOR, IPL2SOFT) IntUart4Handler(void)
{
	BYTE tmphead4;
        BYTE rxdata4;

  // Is this an RX interrupt?
  if (INTGetFlag(INT_SOURCE_UART_RX(UART4)))
    {
        rxdata4 = UARTGetDataByte(UART4);
        // calculate buffer index
        tmphead4 = ( USART_RxHead4 + 1 ) & USART_RX_BUFFER_MASK5;
        // store new index
        USART_RxHead4 = tmphead4;
        // if error
        if ( tmphead4 == USART_RxTail4 )
        {
            UARTEnable(UART4, UART_DISABLE_FLAGS( UART_RX));
            UARTEnable(UART4, UART_ENABLE_FLAGS(UART_RX));
        }
        //store received data in buffer
        USART_RxBuf4[tmphead4] = rxdata4;

      // Clear the RX interrupt Flag
      INTClearFlag(INT_SOURCE_UART_RX(UART4));

      // Toggle LED to indicate UART activity
      bluLED_Toggle;
    }

  // We don't care about TX interrupt
  if ( INTGetFlag(INT_SOURCE_UART_TX(UART4)) )
    {
      INTClearFlag(INT_SOURCE_UART_TX(UART4));
    }
}
//*****************************************************************************
// UART 5 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_5_VECTOR, IPL3SOFT) IntUart5Handler(void)
{
	BYTE tmphead5;
	BYTE rxdata5;

  // Is this an RX interrupt?
  if (INTGetFlag(INT_SOURCE_UART_RX(UART5)))
    {
        rxdata5 = UARTGetDataByte(UART5);
        // calculate buffer index
        tmphead5 = ( USART_RxHead5 + 1 ) & USART_RX_BUFFER_MASK5;
        // store new index
        USART_RxHead5 = tmphead5;
        // if error
        if ( tmphead5 == USART_RxTail5 )
        {
            UARTEnable(UART5, UART_DISABLE_FLAGS( UART_RX));
            UARTEnable(UART5, UART_ENABLE_FLAGS(UART_RX));
        }
        //store received data in buffer
        USART_RxBuf5[tmphead5] = rxdata5;

        // Clear the RX interrupt Flag
        INTClearFlag(INT_SOURCE_UART_RX(UART5));

        // Toggle LED to indicate UART activity
        bluLED_Toggle;
    }

  // We don't care about TX interrupt
  if ( INTGetFlag(INT_SOURCE_UART_TX(UART5)) )
    {
      INTClearFlag(INT_SOURCE_UART_TX(UART5));
    }
}
//******************************************************************
//*	USART2 Receive Character Function
//******************************************************************
BYTE recvchar2(void)
{
  BYTE tmptail2;
  // wait for incomming data 
  while ( USART_RxHead2 == USART_RxTail2 );
  // calculate buffer index 
  tmptail2 = ( USART_RxTail2 + 1 ) & USART_RX_BUFFER_MASK2;
  // store new index 
  USART_RxTail2 = tmptail2;
  return USART_RxBuf2[tmptail2];
}
//******************************************************************
//*	USART4 Receive Character Function
//******************************************************************
BYTE recvchar4(void)
{
  BYTE tmptail4;
  // wait for incomming data
  while ( USART_RxHead4 == USART_RxTail4 );
  // calculate buffer index
  tmptail4 = ( USART_RxTail4 + 1 ) & USART_RX_BUFFER_MASK5;
  // store new index
  USART_RxTail4 = tmptail4;
  return USART_RxBuf4[tmptail4];
}
//******************************************************************
//*	USART5 Receive Character Function
//******************************************************************
BYTE recvchar5(void)
{
  BYTE tmptail5;
  // wait for incomming data
  while ( USART_RxHead5 == USART_RxTail5 );
  // calculate buffer index
  tmptail5 = ( USART_RxTail5 + 1 ) & USART_RX_BUFFER_MASK5;
  // store new index
  USART_RxTail5 = tmptail5;
  return USART_RxBuf5[tmptail5];
}
//******************************************************************
//*	USART2 Character Waiting Function
//******************************************************************
BYTE CharInQueue2(void)
{
  return(USART_RxHead2 != USART_RxTail2);
}
//******************************************************************
//*	USART4 Character Waiting Function
//******************************************************************
BYTE CharInQueue4(void)
{
  return(USART_RxHead4 != USART_RxTail4);
}

//*	USART5 Character Waiting Function
//******************************************************************
BYTE CharInQueue5(void)
{
  return(USART_RxHead5 != USART_RxTail5);
}
//******************************************************************************
//* DELAY MILLISECOND FUNCTIONS
//******************************************************************************
void delayms(unsigned int ms)
{
	unsigned int msDelayTime, currentTickCnt;
	currentTickCnt = ReadCoreTimer(); 
	msDelayTime = (CoreTicksPerMs * ms) + currentTickCnt; 
	while((ReadCoreTimer()) < msDelayTime); 
}
//******************************************************************************
//* UART5 REDIRECTOR
//******************************************************************************
void _mon_putc(char c)
{
	U5TXREG = c;
	while(U5STAbits.TRMT == 0);
}
//******************************************************************************
//* ENTER CMD MODE
//******************************************************************************
BYTE enterCmdMode(void)
{
    unsigned int i;
    BYTE rc;

    rc = 1;
    i = 0;
    bitein = 0;

    delayms(300);
    printf("$$$");
    delayms(300);

    do{
        if(CharInQueue5())
        {
            bitein = recvchar5();
            rxBuf5[i++] = bitein;
        }
    }while(bitein != 0x0A);

    if(rxBuf5[0] != 'C')
        return(rc);
    if(rxBuf5[1] != 'M')
        return(rc);
    if(rxBuf5[2] != 'D')
        return(rc);

    rc = 0;
    return(rc);
}
//******************************************************************************
//* INITIALIZE FUNCTION
//******************************************************************************
void init(void)
 {
    int  value;
    
    AD1PCFG = 0xFFFF;

    TRISBCLR = 0xC020; //1100 0000 0010 0000
    TRISDCLR = 0x000A; //0000 0000 0000 1010
    TRISECLR = 0x003C; //0000 0000 0011 1100
    TRISFCLR = 0x0020; //0000 0000 0010 0000
	
    gpio9LO;
    bluLED_Off;
    redLED_Off;
    grnLED_Off;
   

    value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );

    // Enable the cache for the best performance
    CheKseg0CacheOn();
	// Wait for PLL lock to stabilize
    value = OSCCON;
    while (!(value & 0x00000020))
    {
    value = OSCCON;    
    }
    delayms(50);
    
    DDPCONbits.JTAGEN = 0;

    rstLO;
    delayms(100);
    rstHI;
    delayms(5000);

    //Initialize UART
    setbuf(stdout,NULL);	//UART5 REDIRECT

    //Initialize UART1
    U1BRG = 43; // Set Baud rate 115200
    U1STA = 0;
    U1MODE = 0x00008000; // Enable UART for 8-none-1
    U1STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <4:2> sub bits <0:1>
    IPC6SET = 0x0000000B;
    IEC0SET = 0x08000000;
    IFS0CLR = 0x08000000;
    // flush receive buffer 
    USART1_RxTail = 0x00;
    USART1_RxHead = 0x00;

    //Initialize UART2
    U2BRG = 520; // Set Baud rate 9600
    U2STA = 0;
    U2MODE = 0x00008000; // Enable UART for 8-none-1    
    U2STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <4:2> sub bits <0:1>
    IPC8SET = 0x0000000B;
    IEC1SET = 0x00000200;
    IFS1CLR = 0x00000200;
    // flush receive buffer 
    USART2_RxTail = 0x00;
    USART2_RxHead = 0x00;
   
    //Initialize UART4
    U4BRG = 520; // Set Baud rate 9600
    U4STA = 0;
    U4MODE = 0x00008000; // Enable UART for 8-none-1
    U4STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <12:10> sub bits <9:8>
    IPC12SET = 0x00000B00;
    IEC2SET = 0x00000010;
    IFS2CLR = 0x00000010;
    //flush receive buffer 
    USART4_RxTail = 0x00;
    USART4_RxHead = 0x00;
   
    //Initialize UART5
    U5BRG = 43; // Set Baud rate 115200
    U5STA = 0;
    U5MODESET = 0x00008000; // disable UART for 8-none-1
    U5STASET = 0x00001400; // Enable Transmit and Receive
     //priority 2 sub priority 3 - 00011111 bits <26:28> sub bits <25:24>
    IPC12SET = 0x0B000000;//0000 1011 0000 0000 0000 0000 0000 0000
    IEC2SET = 0x00000400;
    IFS2CLR = 0x00000400;
    //flush receive buffer 
    USART5_RxTail = 0x00;
    USART5_RxHead = 0x00;

    // Enable Interrupts
    INTEnableSystemMultiVectoredInt();

    // enable interrupts
    INTEnableInterrupts();
 }
//******************************************************************************
// MAIN
//******************************************************************************
int main (void)
{
	init();
	
	//enter COMMAND mode from STREAM mode
	grnLED_On;
	delayms(300);
	printf("$$$");
	delayms(300);
	
	//wait for Command Mode Start message - ends with >
	indx5 = 0;
	while(!CharInQueue5());
	//if(CharInQueue5())
	//{
		do{
			rxBuf5[indx5++] = recvchar5();
		}while(CharInQueue5());
	//}
	//issue tcp client command
	if(rxBuf5[20] == '>')
	{
		printf("tcp_client nemo.dyndns-server.com 8032\r\n");
		delayms(1000);
	}
	//exit to STREAM mode
	printf("exit\r\n");
    //forward USART4 traffic to server
    do{
    if(CharInQueue4())
     {
         U5TXREG = recvchar4();
         while(U5STAbits.TRMT == 0);
     }
    }while(1);
}


